SystemOrganization addCategory: #'Collections-Graphs'!

PackageInfo subclass: #GraphInfo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Graphs'!

!GraphInfo methodsFor: 'as yet unclassified' stamp: 'Samir Saidani 9/17/2003 18:08'!
prerequisiteChangeSets
	"These are the names of DVS packages that get loaded before the main  
	change set"
	^ #('Collections-Misc' 'Iterator' )! !

Object subclass: #Dijkstra
	instanceVariableNames: 'graph source predecessor distance '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Graphs'!

!Dijkstra methodsFor: 'as yet unclassified' stamp: 'len 2/15/2003 18:52'!
graph: aGraph source: aNode
	graph _ aGraph.
	source _ aNode.
	self run! !

!Dijkstra methodsFor: 'as yet unclassified' stamp: 'len 2/13/2003 21:55'!
shortestPathTo: aGraphNode
	| answer node predecessorNode |
	answer _ OrderedCollection new.
	[node _ aGraphNode.
	answer add: node.
	predecessorNode _ predecessorNode at: node.
	source = predecessorNode]
		whileFalse: [node _ predecessorNode].
	^ answer! !

!Dijkstra methodsFor: 'as yet unclassified' stamp: 'len 2/13/2003 21:56'!
distanceTo: aGraphNode
	^ distance at: aGraphNode! !

!Dijkstra methodsFor: 'as yet unclassified' stamp: 'len 2/13/2003 21:56'!
eccentricity
	"Answer the eccentricity of the source vertex.
	The eccentricity of a vertex is the length of the longest minimal path from that vertex to some vertex in the graph. You can think of the eccentricity of a vertex as the longest distance in the graph from there to somewhere."

	^ distance max! !

!Dijkstra methodsFor: 'as yet unclassified' stamp: 'len 2/13/2003 21:56'!
run
	| queue u estimate |
	graph do: [:each |
		distance at: each put: Float infinity.
		predecessor at: each put: nil].
	distance at: source put: 0.

	queue _ Heap sortBlock: [:a :b | (distance at: a) <= (distance at: b)].
	queue addAll: graph.
	[queue isEmpty]
		whileFalse: [u _ queue removeFirst.
					(graph nodeFor: u) neighborsAndLabelsDo: [:each :weight |
						estimate _ (distance at: u) + (weight ifNil: [1]).
						(distance at: each) > estimate
							ifTrue: [distance at: each put: estimate.
									predecessor at: each put: u]]]! !

Object subclass: #GraphNode
	instanceVariableNames: 'value '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Graphs'!

!GraphNode commentStamp: '<historical>' prior: 0!
GraphNode is an abstract class of nodes that are held in a graph.

Each node holds on to a corresponding object that is the value of that node.

Subclasses add state/behaviour to represent edges in the graph.!

!GraphNode methodsFor: 'enumerating' stamp: 'len 2/14/2003 20:05'!
walkPre: preBlock post: postBlock
	"Recursively walk the subtree rooted at me.  Apply preBlock to each node, then walk the subtree below node, then apply postBlock to the node."
	preBlock value: self.
	self neighborsDo: [ :child | child walkPre: preBlock post: postBlock].
	postBlock value: self! !

!GraphNode methodsFor: 'accessing value' stamp: ''!
value
	^value! !

GraphNode subclass: #ExplicitGraphNode
	instanceVariableNames: 'neighbors '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Graphs'!

!ExplicitGraphNode commentStamp: '<historical>' prior: 0!
An ExplicitGraphNode stores the edges in a collection, an instance variable of self.
The type of the collection varies as to whether the graph is ordered, of fixed arity, etc.

To change the edge collection, addNeighbor: and removeNeighbor: messages are supplied.
!

!ExplicitGraphNode methodsFor: 'testing' stamp: ''!
isLabelled
	"Are the edges from this node labelled?"
	^false! !

!ExplicitGraphNode methodsFor: 'testing' stamp: ''!
isOrdered
	^neighbors isSequenceable! !

!ExplicitGraphNode class methodsFor: 'instance creation' stamp: 'len 2/12/2003 21:40'!
newOn: value
	"Create a new unordered node, and given value."
	^super new initializeUnordered; value: value! !

!ExplicitGraphNode methodsFor: 'accessing edges' stamp: ''!
degree
	^neighbors size! !

ExplicitGraphNode subclass: #LabelledExplicitGraphNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Graphs'!

!LabelledExplicitGraphNode commentStamp: '<historical>' prior: 0!
A LabelledExplicitGraphNode associates a label with each neighbor.

When adding neighbors, the label must be given.!

!LabelledExplicitGraphNode methodsFor: 'accessing edges' stamp: ''!
labelOfEdgeTo: anObject ifNone: aBlock
	"Return the label associated with the edge to anObject; return the value of the block is no such edge exists."
	^(neighbors detect: [ :assoc | assoc key = anObject] ifNone: [^aBlock value]) value! !

!LabelledExplicitGraphNode methodsFor: 'changing edges' stamp: ''!
addNeighbor: aNode at: index label: label
	"pre: self isOrdered"
	^super addNeighbor: aNode -> label at: index! !

!LabelledExplicitGraphNode methodsFor: 'changing edges' stamp: ''!
addNeighbor: aNode label: label
	^super addNeighbor: aNode -> label! !

!LabelledExplicitGraphNode methodsFor: 'accessing edges' stamp: ''!
neighborsAndLabelsDo: aBlock
	"Evaluate aBlock for each node connected to an outgoing edge, and the label on that edge (nil if no label)."
	^neighbors do: [ :assoc | aBlock value: assoc key value: assoc value]! !

!LabelledExplicitGraphNode methodsFor: 'accessing edges' stamp: ''!
labelOfEdgeTo: anObject
	"Return the label associated with the edge to anObject; otherwise signal an error."
	^(neighbors detect: [ :assoc | assoc key = anObject]) value! !

!LabelledExplicitGraphNode methodsFor: 'testing edges' stamp: ''!
hasEdgeTo: anObject
	^neighbors anySatisfy: [ :assoc | assoc key = anObject]! !

!LabelledExplicitGraphNode methodsFor: 'testing' stamp: ''!
isLabelled
	"Are the edges from this node labelled?"
	^true! !

!LabelledExplicitGraphNode methodsFor: 'changing edges' stamp: ''!
removeNeighbor: aNode
	neighbors remove: (neighbors detect: [ :assoc | assoc key = aNode])! !

!LabelledExplicitGraphNode methodsFor: 'accessing edges' stamp: ''!
neighbors
	^neighbors collect: [ :assoc | assoc key]! !

!LabelledExplicitGraphNode methodsFor: 'accessing edges' stamp: ''!
neighborsDo: aBlock
	^neighbors do: [ :assoc | aBlock value: assoc key]! !

!LabelledExplicitGraphNode methodsFor: 'changing edges' stamp: ''!
addNeighbor: aNode at: index
	self shouldNotImplement! !

!LabelledExplicitGraphNode methodsFor: 'changing edges' stamp: ''!
addNeighbor: aNode
	self shouldNotImplement! !

!ExplicitGraphNode class methodsFor: 'instance creation' stamp: 'len 2/12/2003 21:40'!
ordered: value
	"Create a new ordered node, and given value."
	^super new initializeOrdered; value: value! !

!ExplicitGraphNode class methodsFor: 'instance creation' stamp: ''!
binary
	"Create a new binary node."
	^super new initializeFixed: 2! !

!ExplicitGraphNode methodsFor: 'changing edges' stamp: ''!
addNeighbor: aNode 
	(neighbors includes: aNode)
		ifFalse: [neighbors add: aNode.
			self triggerEvent: #addNeighbor: with: aNode].
	^ aNode! !

!ExplicitGraphNode methodsFor: 'initialize' stamp: ''!
initializeUnordered
	neighbors := Set new! !

!ExplicitGraphNode methodsFor: 'initialize' stamp: ''!
initializeFixed: arity
	neighbors := Array new: arity! !

!ExplicitGraphNode methodsFor: 'changing edges' stamp: 'len 2/12/2003 21:41'!
addNeighbor: aNode at: index
	"pre: self isOrdered"
	^ neighbors at: index put: aNode! !

!ExplicitGraphNode methodsFor: 'changing edges' stamp: 'len 2/12/2003 21:41'!
removeNeighbor: aNode
	^ neighbors remove: aNode! !

!ExplicitGraphNode class methodsFor: 'instance creation' stamp: 'len 2/12/2003 21:40'!
binary: value
	"Create a new binary node, and given value."
	^super new initializeFixed: 2; value: value! !

!ExplicitGraphNode methodsFor: 'accessing edges' stamp: ''!
neighborsDo: aBlock
	neighbors do: aBlock! !

!ExplicitGraphNode methodsFor: 'copying' stamp: 'len 2/12/2003 21:43'!
copy
	^ super copy setNeighbors: neighbors copy! !

!ExplicitGraphNode methodsFor: 'testing edges' stamp: ''!
hasEdgeTo: anObject
	^neighbors includes: anObject! !

!ExplicitGraphNode methodsFor: 'private' stamp: ''!
setNeighbors: aCollection
	neighbors := aCollection! !

!ExplicitGraphNode class methodsFor: 'instance creation' stamp: ''!
ordered
	"Create a new ordered node."
	^super new initializeOrdered! !

!ExplicitGraphNode class methodsFor: 'instance creation' stamp: ''!
arity: n
	"Create a new ordered node with fixed arity n."
	^super new initializeFixed: n! !

!ExplicitGraphNode methodsFor: 'accessing edges' stamp: ''!
neighbors
	^neighbors! !

!ExplicitGraphNode class methodsFor: 'instance creation' stamp: ''!
new
	"Create a new unordered node."
	^super new initializeUnordered! !

!ExplicitGraphNode class methodsFor: 'instance creation' stamp: 'len 2/12/2003 21:40'!
on: value arity: n
	"Create a new ordered node with fixed arity n, and given value."
	^super new initializeFixed: n; value: value! !

!ExplicitGraphNode methodsFor: 'initialize' stamp: ''!
initializeOrdered
	neighbors := OrderedCollection new! !

!GraphNode methodsFor: 'enumerating' stamp: 'len 2/18/2003 21:37'!
markDo: aBlock
	"Visit each node in the graph once, applying aBlock.
	A node is only visited after at least one of its predecessors, but not necessarily after all the predecessors."
	| todo visited |
	todo _ Set with: self.
	visited := Set new.
	[todo isEmpty] whileFalse:
		[| node |
		node := todo anyOne.
		visited add: node.
		aBlock value: node.
		node neighborsDo:
			[ :child |
				(visited includes: child)
					ifFalse: [todo add: child]].
		todo remove: node]! !

!GraphNode methodsFor: 'testing' stamp: ''!
isLabelled
	"Are the edges from this node labelled?"
	self subclassResponsibility! !

!GraphNode methodsFor: 'comparing' stamp: ''!
= anObject
	"Comparison is delegated to the value of the node.
	Argument and receiver are reversed to dereference through other GraphNodes."
	^anObject = value! !

!GraphNode methodsFor: 'accessing value' stamp: ''!
value: val
	value := val! !

!GraphNode methodsFor: 'accessing edges' stamp: ''!
neighbors
	"Return a collection of nodes connected to outgoing edges."
	self subclassResponsibility! !

!GraphNode methodsFor: 'accessing edges' stamp: ''!
neighborsDo: aBlock
	"Evaluate aBlock for each node connected to an outgoing edge."
	self subclassResponsibility! !

!GraphNode methodsFor: 'accessing edges' stamp: ''!
degree
	"How many edges does this node have?"
	self subclassResponsibility! !

!GraphNode methodsFor: 'printing' stamp: 'len 2/14/2003 23:38'!
printOn: aStream
	aStream nextPut: $[; print: value; nextPut: $]! !

GraphNode subclass: #ImplicitGraphNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Graphs'!

!ImplicitGraphNode commentStamp: '<historical>' prior: 0!
An ImplicitGraphNode has the edge structure represented within the value, and therefore forwards messages to the value to access edges.

To change the edge collection, you must send messages to the value.

Subclasses decide as to whether the value can yield a collection of edges more efficiently than iterating over them.!

!ImplicitGraphNode class methodsFor: 'instance creation' stamp: ''!
on: aValue iterator: anIterator
	^ImplicitGraphNodeIterator on: aValue iterator: anIterator! !

!ImplicitGraphNode class methodsFor: 'instance creation' stamp: ''!
on: aValue collectionBlock: aBlock
	^ImplicitGraphNodeCollection on: aValue collectionBlock: aBlock! !

!ImplicitGraphNode methodsFor: 'accessing edges' stamp: ''!
neighborsDo: aBlock
	self neighbors do: aBlock! !

!ImplicitGraphNode methodsFor: 'accessing edges' stamp: ''!
neighbors
	^OrderedCollection accumulate: [ :incBlock | self neighborsDo: incBlock]! !

!ImplicitGraphNode class methodsFor: 'instance creation' stamp: ''!
on: aValue iteratorBlock: iteratorBlock
	^ImplicitGraphNodeIterator on: aValue iteratorBlock: iteratorBlock! !

ImplicitGraphNode subclass: #ImplicitGraphNodeIterator
	instanceVariableNames: 'iterator '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Graphs'!

!ImplicitGraphNodeIterator commentStamp: '<historical>' prior: 0!
I can iterate over the collection of edges by evaluating the iterator.!

!ImplicitGraphNodeIterator methodsFor: 'initialisation' stamp: ''!
iterator: anIterator
	iterator := anIterator! !

!ImplicitGraphNodeIterator methodsFor: 'testing edges' stamp: ''!
hasEdgeTo: anObject
	^iterator includes: anObject! !

!ImplicitGraphNodeIterator class methodsFor: 'instance creation' stamp: ''!
on: aValue iteratorBlock: iteratorBlock
	^self on: aValue iterator: (Iterator on: iteratorBlock)! !

!ImplicitGraphNodeIterator methodsFor: 'accessing edges' stamp: ''!
degree
	^iterator size! !

!ImplicitGraphNodeIterator class methodsFor: 'instance creation' stamp: ''!
on: aValue iterator: anIterator
	^(self new) value: aValue ; iterator: anIterator! !

!ImplicitGraphNodeIterator methodsFor: 'accessing edges' stamp: ''!
neighborsDo: aBlock
	iterator do: aBlock! !

ImplicitGraphNode subclass: #ImplicitGraphNodeCollection
	instanceVariableNames: 'collectionBlock '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Graphs'!

!ImplicitGraphNodeCollection commentStamp: '<historical>' prior: 0!
I access the collection of edges by evaluating my block.!

!ImplicitGraphNodeCollection methodsFor: 'accessing edges' stamp: ''!
degree
	^self neighbors size! !

!ImplicitGraphNodeCollection class methodsFor: 'instance creation' stamp: ''!
on: aValue collectionBlock: aBlock
	^(self new) value: aValue ; collectionBlock: aBlock! !

!ImplicitGraphNodeCollection methodsFor: 'testing edges' stamp: ''!
hasEdgeTo: anObject
	^self neighbors includes: anObject! !

!ImplicitGraphNodeCollection methodsFor: 'accessing edges' stamp: ''!
neighbors
	^collectionBlock value! !

!ImplicitGraphNodeCollection methodsFor: 'initialisation' stamp: ''!
collectionBlock: aBlock
	collectionBlock := aBlock! !

!GraphNode methodsFor: 'comparing' stamp: ''!
hash
	^value hash! !

!GraphNode methodsFor: 'testing edges' stamp: ''!
hasEdgeTo: anObject
	"Is there an edge from this node to anObject?"
	self subclassResponsibility! !

!GraphNode methodsFor: 'testing edges' stamp: ''!
isLeaf
	^self degree = 0! !

!GraphNode methodsFor: 'accessing edges' stamp: ''!
neighborsAndLabelsDo: aBlock
	"Evaluate aBlock for each node connected to an outgoing edge, and the label on that edge (nil if no label)."
	"The default definition assumes no labels are present."
	self neighborsDo: [ :n | aBlock value: n value: nil]! !

Object subclass: #DAGFrontier
	instanceVariableNames: 'frontier bag '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Graphs'!

!DAGFrontier commentStamp: '<historical>' prior: 0!
A DAGFrontier is used to enumerate a directed acyclic graph (DAG) respecting the dependencies between nodes.  A client does not create it directly, but sends the message "frontier" to the DAG in question.

At any moment it holds a "frontier" of nodes that can be accessed.  When any node in the frontier has been processed, it can be removed from the frontier, and the frontier will be updated if necessary (in situ).

Instance variables:
frontier		<Set> of nodes on the frontier, empty if the whole DAG has been enumerated.
bag		<Bag> of nodes, used to calculate updates to the frontier.

Clients of this class must *not* modify the frontier set, only access it.
!

!DAGFrontier methodsFor: 'initialization' stamp: ''!
frontier: f bag: remaining
	frontier := f.
	bag := remaining! !

!DAGFrontier class methodsFor: 'instance creation' stamp: ''!
frontierOn: rootedGraph
	"Create a frontier object to enumerate the graph."
	| remaining frontier |
	"Build a Bag of non-root nodes, each node once in the Bag for every predecessor."
	frontier := rootedGraph rootNodes asOrderedCollection.
	remaining := Bag new.
	rootedGraph nodesDo: [ :node | node neighborsDo: [ :n | remaining add: n]].
	^self new frontier: frontier bag: remaining! !

!DAGFrontier methodsFor: 'advancing' stamp: ''!
removeAll: collection
	collection do: [ :n | self remove: n]! !

!DAGFrontier methodsFor: 'accessing' stamp: ''!
frontier
	^frontier! !

!DAGFrontier methodsFor: 'advancing' stamp: ''!
remove: frontierObject
	frontier remove: frontierObject.
	frontierObject neighborsDo:
		[ :neighbor |
			(bag removeOne: neighbor) = 0
				ifTrue: [frontier add: neighbor]].
	^frontierObject! !

Collection subclass: #Graph
	instanceVariableNames: 'nodes nodeCreator type '
	classVariableNames: 'InitializationBlocks '
	poolDictionaries: ''
	category: 'Collections-Graphs'!

!Graph commentStamp: '<historical>' prior: 0!
A Graph is a collection of GraphNodes.

Various properties of the graph are encoded in the class of its nodes (eg whether the graph is ordered, has a fixed arity, or the edges or labelled).
The creation block is initialized so that appropriate nodes are made.

Instance variable:
nodes:	a KeyedSet of GraphNodes
nodeCreator: a block which is evaluated to create a new node.
type:	a Symbol indicating what the default node type is.

PS: It's still not clear to me whether there's much to be gained by having this under Collection.  Little is inherited; much of what is inherited has no obvious interpretation or meaning, so why bother?  Problems, problems...!

!Graph class methodsFor: 'examples' stamp: ''!
exampleImplicitGraph
	"Graph exampleImplicitGraph."
	| tree |
	tree := self implicitCollection: [ :class | class subclasses].
	Number withAllSubclasses do: [ :class | tree addNode: class].
	^tree! !

!Graph methodsFor: 'accessing' stamp: 'Samir Saidani 6/19/2003 20:17'!
type
	^type! !

!Graph methodsFor: 'testing' stamp: ''!
occurrencesOf: anObject 
	"Answer how many of the receiver's elements are equal to anObject."

	^(self includes: anObject) ifTrue: [1] ifFalse: [0]! !

!Graph methodsFor: 'accessing' stamp: ''!
numberOfEdges
	"This is commonly known as the size of the graph, but the size message returns the order (number of vertices), as this fits in better with Smalltalk usage."
	^(Iterator on: self msg: #edgesDo:) size! !

!Graph methodsFor: 'enumerating' stamp: ''!
edgesDo: aBlock
	self nodesDo: [ :node |
		node neighborsDo: [ :n |
			aBlock value: node -> n]]! !

!Graph class methodsFor: 'instance creation' stamp: ''!
arity: n
	"Create a new ordered graph with fixed arity."
	^super new initialize arity: n! !

!Graph methodsFor: 'errors' stamp: 'len 2/13/2003 23:29'!
notFoundError
	^ self error: 'object is not in the collection'! !

!Graph methodsFor: 'copying' stamp: ''!
copyEmpty
	"Return a graph of the same type with no nodes or edges."
	^self isImplicit
		ifTrue: [self class ordered]
		ifFalse: [self copy initialize]! !

!Graph methodsFor: 'initialization' stamp: ''!
unordered
	type := #(#unordered).
	nodeCreator := InitializationBlocks at: #unordered! !

!Graph methodsFor: 'initialization' stamp: ''!
undirected
	type _ #(#undirected ).
	nodeCreator _ InitializationBlocks at: #undirected! !

!Graph methodsFor: 'accessing' stamp: ''!
getNode: aSymbol 
	^ self nodes at: aSymbol! !

!Graph methodsFor: 'initialization' stamp: ''!
ordered
	type := #(#ordered).
	nodeCreator := InitializationBlocks at: #ordered! !

!Graph class methodsFor: 'initialization' stamp: ''!
initialize
	"Graph initialize"
	| d |
	d _ Dictionary new.
	d
		at: #binary
		put: [:value | ExplicitGraphNode binary: value].
	d
		at: #arity
		put: [:n | [:value | ExplicitGraphNode on: value arity: n]].
	d
		at: #binaryLabelled
		put: [:value | LabelledExplicitGraphNode binary: value].
	d
		at: #ordered
		put: [:value | ExplicitGraphNode ordered: value].
	d
		at: #unordered
		put: [:value | ExplicitGraphNode newOn: value].
d
		at: #undirected
		put: [:value | ExplicitGraphNode newOn: value].
	d
		at: #arityLabelled
		put: [:n | [:value | LabelledExplicitGraphNode on: value arity: n]].
	d
		at: #orderedLabelled
		put: [:value | LabelledExplicitGraphNode ordered: value].
	d
		at: #unorderedLabelled
		put: [:value | LabelledExplicitGraphNode newOn: value].
	d
		at: #implicitCollection
		put: [:graph :collectionBlock | [:value | ImplicitGraphNode
				on: value
				collectionBlock: [(collectionBlock value: value)
						collect: [:node | graph addNode: node]]]].
	d
		at: #implicitIteratorBlock
		put: [:graph :iteratorBlock | [:value | 
			| iterator | 
			iterator _ iteratorBlock value: value.
			ImplicitGraphNode
				on: value
				iteratorBlock: [:aBlock | iterator
						value: [:node | aBlock
								value: (graph addNode: node)]]]].
	InitializationBlocks _ d! !

!Graph methodsFor: 'private' stamp: ''!
isImplicit
	"Is the node creator for implicit nodes?"
	^(type includes: #implicitCollection) or: [type includes: #implicitIteratorBlock]! !

!Graph class methodsFor: 'instance creation' stamp: ''!
arityLabelled: n
	"Create a new labelled, ordered graph with fixed arity."
	^super new initialize arityLabelled: n! !

!Graph class methodsFor: 'instance creation' stamp: ''!
implicitIteratorBlock: iteratorBlock
	"Create a new graph, using the structure implicit in existing objects.
	Each node is the graph is accessed by evaluating iteratorBlock to iterate over a collection of neighbouring nodes."
	^super new initialize implicitIteratorBlock: iteratorBlock! !

!Graph methodsFor: 'adding' stamp: ''!
addEdgeFrom: source to: target label: label
	"pre: (self nodeFor: edge key) isLabelled"
	| src trgt |
	src := self addNode: source.
	trgt := self addNode: target.
	src addNeighbor: trgt label: label! !

!Graph class methodsFor: 'private' stamp: ''!
new
	"^ self error: 'Graphs should not be created with new.'"
^super new.! !

!Graph class methodsFor: 'instance creation' stamp: 'Samir Saidani 6/19/2003 17:30'!
undirected
	"Create an undirected graph"
	^ super new initialize undirected.! !

!Graph methodsFor: 'graph operations' stamp: 'len 2/13/2003 22:09'!
symmetric
	| answer |
	answer _ self copy.
	self edgesDo: [:each | answer addEdge: each value -> each key].
	^ answer! !

!Graph methodsFor: 'removing' stamp: ''!
remove: aGraphNode 
	"Remove the given node.
	pre: the node must be in the graph, and not the target of an edge."
	^self remove: aGraphNode ifAbsent: [self notFoundError]! !

!Graph methodsFor: 'removing' stamp: ''!
removeEdge: edge ifAbsent: exceptionBlock 
	"Remove the given edge, evaluate exceptionBlock if it doesn't exist.
	pre: both the source and target of the edge must be in the graph."
	^self
		removeEdgeFrom: edge key
		to: edge value
		ifAbsent: exceptionBlock! !

!Graph methodsFor: 'enumerating' stamp: ''!
nodesDo: aBlock
	"No particular order."
	nodes do: aBlock! !

!Graph methodsFor: 'adding' stamp: ''!
addEdge: edge 
	"Add an edge (represented by an Association)"
	self addEdgeFrom: edge key to: edge value.
	self triggerEvent: #addEdge: with: edge.
	type = #(#undirected )
		ifTrue: [| reversedEdge | 
			reversedEdge _ edge value -> edge key.
			self addEdgeFrom: edge value to: edge key.
			self triggerEvent: #addEdge: with: reversedEdge].
	^ edge! !

!Graph methodsFor: 'initialization' stamp: ''!
arityLabelled: n
	type := #(#arity #labelled).
	nodeCreator := (InitializationBlocks at: #arityLabelled) value: n! !

!Graph methodsFor: 'graph operations' stamp: 'len 2/14/2003 22:56'!
components
	"Answer the Strongly Connected Components of the receiver."
	^ Set accumulate: [:aBlock | self componentsDo: aBlock]! !

!Graph methodsFor: 'enumerating' stamp: ''!
valuesDo: aBlock
	"No particular order."
	nodes do: [ :node | aBlock value: node value]! !

!Graph class methodsFor: 'instance creation' stamp: ''!
orderedLabelled
	"Create a new labelled, ordered graph."
	^super new initialize orderedLabelled! !

!Graph class methodsFor: 'instance creation' stamp: ''!
implicitCollection: collectionBlock
	"Create a new graph, using the structure implicit in existing objects.
	Each node is the graph is accessed by evaluating collectionBlock to yield a collection of neighbouring nodes."
	^super new initialize implicitCollection: collectionBlock! !

!Graph methodsFor: 'adding' stamp: ''!
addNode: aSymbolNode 
	"Create a new GraphNode for the value (if necessary), and add it to the  
	graph. You shouldn't be calling this unless you're sure what you're  
	doing. Better to add nodes by adding edges."
	^ nodes
		at: aSymbolNode
		ifAbsent: [self
				add: (self newNodeOn: aSymbolNode).
			self
				triggerEvent: #addNode:
				with: (self getNode: aSymbolNode).
				 nodes at: aSymbolNode.]! !

!Graph methodsFor: 'accessing' stamp: ''!
edges
	^OrderedCollection accumulate: [ :incBlock | self edgesDo: incBlock]! !

!Graph methodsFor: 'initialization' stamp: ''!
orderedLabelled
	type := #(#ordered #labelled).
	nodeCreator := InitializationBlocks at: #orderedLabelled! !

!Graph methodsFor: 'initialization' stamp: ''!
arity: n
	type := #(#arity).
	nodeCreator := (InitializationBlocks at: #arity) value: n! !

Graph subclass: #RootedGraph
	instanceVariableNames: 'roots '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Graphs'!

!RootedGraph commentStamp: '<historical>' prior: 0!
A RootedGraph is a directed graph with roots.

Instance variable:
roots:	Set of root objects!

!RootedGraph methodsFor: 'enumerating' stamp: ''!
preOrderDo: aBlock
	"Walk the graph, evaluating aBlock for each node.  Evaluate each node before its successors, but in no particular order when there is a choice.
	pre: self isCyclic not"
	| frontier remaining | 
	frontier := self rootNodes.
	"Build a Bag of non-root nodes, each node once in the Bag for every predecessor."
	remaining := Bag new.
	nodes do: [ :node | remaining addAll: node neighbors].
	[| next |
	next := frontier anyIfNone: [^self].
	aBlock value: next.
	frontier remove: next.
	next neighborsDo:
		[ :neighbor |
			(remaining removeOne: neighbor) = 0
				ifTrue: [frontier add: neighbor]]
	] repeat! !

!RootedGraph methodsFor: 'enumerating' stamp: ''!
breadthFirstDo: aBlock
	"Evaluate aBlock for each set of nodes at a given depth in the graph.
	A node is at depth n if the longest path to that node from a root has n steps.
	pre: self isCyclic not"

	| df current |
	df := self frontier.
	[current := df frontier copy.
	current isEmpty] whileFalse:
		[aBlock value: current.
		df removeAll: current]! !

!RootedGraph methodsFor: 'copying' stamp: ''!
copyNodes: nodeBlock  labels: labelBlock
	"Make a copy with the same number of nodes and edges.
	Each node is self is mapped though nodeBlock, labels (if used) are
	mapped through labelBlock (if non-nil)."
	| newGraph map |
	newGraph := self copyEmpty.
	map := DictionaryWithDefault newWithDefaultValueBlock: nodeBlock.
	newGraph roots: (roots collect: [ :root | map at: root]).
	self do: [ :node |
		node neighborsAndLabelsDo: [ :n :label |
			(labelBlock notNil and: [label notNil])
				ifTrue: [newGraph 
							addEdgeFrom: (map at: node value)
							to: (map at: n value)
							label: (labelBlock value: label)]
				ifFalse: [newGraph
							addEdgeFrom: (map at: node value)
							to: (map at: n value)]]].
	^newGraph! !

!RootedGraph methodsFor: 'adding' stamp: ''!
addEdgeFrom: source to: target label: label
	super addEdgeFrom: source to: target label: label.
	(roots includes: target) ifTrue: [roots remove: target ifAbsent: []]! !

!RootedGraph class methodsFor: 'examples' stamp: 'len 2/18/2003 22:09'!
exampleSmallTree2
	"RootedGraph exampleSmallTree2"
	^self unordered roots: (Set with: Magnitude) childrenMsg: #subclasses! !

!RootedGraph methodsFor: 'graph operations' stamp: ''!
reduce
	"Remove all redundant edges to form the transitive reduction.

	Based on Algorithm 4 in 'An Algorithm for Finding a Minimum Equivalent Graph of a Digraph', by D. M. Moyles and G. L. Thompson, JACM 16(3), July 1969, pp455-60 (with obvious simplifications for acyclic graphs)."
	
	self rootNodes do: [ :root | self reduceStep: root with: OrderedCollection new]! !

!RootedGraph methodsFor: 'initialization' stamp: ''!
roots: aCollection
	roots := aCollection.
	roots do: [ :root | self addNode: root]! !

!RootedGraph methodsFor: 'testing' stamp: ''!
isEmpty
	"Answer whether the receiver contains any elements."

	^roots isEmpty! !

!RootedGraph methodsFor: 'adding' stamp: 'len 2/18/2003 21:38'!
roots: rootNodes childrenLabelGenerator: aBlock
	"Build the graph with given rootNodes.
	Evaluate aBlock for each node to obtain an iterator over a Collection of its children and labels."
	| todo |
	self roots: rootNodes.
	todo := rootNodes asSet.
	[todo isEmpty] whileFalse:
		[| node |
		node := todo anyOne.
		(aBlock value: node) value:
			[ :child :label |
				(self includes: child) ifFalse: [todo add: child].
				self addEdgeFrom: node to: child label: label].
			todo remove: node].
	^self! !

!RootedGraph methodsFor: 'removing' stamp: 'len 2/14/2003 20:06'!
removeAllFrom: aNode
	"Remove the subtree below aNode (and associated edges), excluding aNode itself.
	Note that nodes are removed from the graph without being sent any messages."
	aNode
		walkPre: [ :junk ]
		post: [ :child |
			child == aNode
				ifTrue: [aNode neighbors copy do: [ :n | aNode removeNeighbor: n]]
				ifFalse: [self remove: child]]! !

!RootedGraph methodsFor: 'testing' stamp: ''!
isTree
	"Is the graph a tree (more accurately, a forest)?
	It is if every node has at most one predecessor."
	| visited |
	visited := Set new.
	nodes do: [ :node |
		node neighborsDo: [ :neighbor |
			(visited includes: neighbor) ifTrue: [^false].
			visited add: neighbor]].
	^true! !

!RootedGraph class methodsFor: 'examples' stamp: 'len 2/12/2003 22:59'!
exampleMediumLabelledTree
	"RootedGraph exampleMediumLabelledTree"
	^self unorderedLabelled
		roots: (Set with: Collection)
		children: [ :class | class subclasses]
		label: [ :src :trg |
				trg isVariable
					ifTrue: [trg isBits ifTrue: [#variableByte] ifFalse: [#variable]]
					ifFalse: [#normal]]! !

!RootedGraph methodsFor: 'initialization' stamp: 'len 2/12/2003 22:29'!
findRoots
	"Find all nodes that are not the targets of an edge and make them the roots."
	roots := Set new addAll: nodes; yourself.
	self nodesDo: [:node |
		node neighborsDo: [ :nb |
			roots remove: nb ifAbsent: []]]! !

!RootedGraph class methodsFor: 'examples' stamp: ''!
exampleImplicitLargeTree
	"RootedGraph exampleImplicitLargeTree"
	^(self
		implicitCollection:
			[ :class |
				class == Class
					ifTrue: [#()]
					ifFalse: [class subclasses asSortedCollection: [ :x :y | x name < y name]]]) 		buildFromRoots: (Array with: Object)! !

!RootedGraph methodsFor: 'adding' stamp: ''!
roots: rootNodes childrenMsg: msg
	"Build the graph with given rootNodes.
	Send msg to each node to obtain a Collection of its children."
	^self roots: rootNodes childrenGenerator:
		[ :node | [ :iterationBlock | (node perform: msg) do: iterationBlock]]! !

!RootedGraph class methodsFor: 'examples' stamp: 'len 2/14/2003 22:58'!
exampleMediumTree
	^self unordered roots: (Set with: Stream) children: [ :class | class subclasses]! !

!RootedGraph class methodsFor: 'examples' stamp: ''!
exampleDAG
	"RootedGraph exampleDAG."
	| d |
	d := self unordered.
	d addRoot: #r1.
	d addRoot: #r2.
	d addEdge: #r1 -> #n1.
	d addEdge: #r1 -> #n2.
	d addEdge: #r2 -> #n2.
	d addEdge: #n1 -> #n3.
	d addEdge: #n2 -> #n3.
	^d! !

!RootedGraph methodsFor: 'graph operations' stamp: ''!
reduceStep: node with: sofar
	node degree = 0 ifTrue: [^self].
	node neighbors copy do: [ :succ |
		sofar do: [ :done |
			(done hasEdgeTo: succ)
				ifTrue: [self removeEdgeFrom: done to: succ]].
		sofar addLast: node.
		self reduceStep: succ with: sofar.
		sofar removeLast]! !

!RootedGraph class methodsFor: 'examples' stamp: 'len 2/12/2003 22:59'!
exampleForest
	"RootedGraph exampleForest"
	^self unordered roots: (Set with: Number with: Stream) children: [ :class | class subclasses]! !

!RootedGraph methodsFor: 'graph operations' stamp: ''!
depthList
	"Partitions the DAG, returning a SequenceableCollection of Sets of nodes in the graph.  Element n in the SequenceableCollection is the Set of nodes in the graph reachable in n-1 steps by the longest route from a root."
	^OrderedCollection accumulate: [ :incBlock | self breadthFirstDo: incBlock]! !

!RootedGraph methodsFor: 'adding' stamp: ''!
roots: rootNodes children: aBlock
	"Build the graph with given rootNodes.
	Evaluate aBlock for each node to obtain a Collection of its children."
	^self roots: rootNodes childrenGenerator:
		[ :node | [ :iterationBlock | (aBlock value: node) do: iterationBlock]]! !

!RootedGraph methodsFor: 'enumerating' stamp: ''!
frontier
	"Return a DAGFrontier object, capable of enumerating this DAG."
	^DAGFrontier frontierOn: self! !

!RootedGraph class methodsFor: 'examples' stamp: 'len 2/12/2003 23:00'!
exampleSmallLabelledTree
	"RootedGraph exampleSmallLabelledTree"
	^self unorderedLabelled
		roots: (Set with: Magnitude)
		children: [ :class | class subclasses]
		label: [ :src :trg | trg category]! !

!RootedGraph methodsFor: 'copying' stamp: ''!
copyEmpty
	"Return a new graph of the same type, with the same roots, but no edges."
	| copy |
	copy := super copyEmpty.
	copy roots: (roots copyEmpty: roots size).
	^copy! !

!RootedGraph class methodsFor: 'examples' stamp: 'len 2/12/2003 22:58'!
exampleLargeTree
	"RootedGraph exampleLargeTree"
	^self ordered roots: (Array with: Object) children:
		[ :class | class == Class
					ifTrue: [#()]
					ifFalse: [class subclasses asSortedCollection: [ :x :y | x name < y name]]]! !

!RootedGraph methodsFor: 'adding' stamp: ''!
addEdgeFrom: source to: target
	super addEdgeFrom: source to: target.
	(roots includes: target) ifTrue: [roots remove: target ifAbsent: []]! !

!RootedGraph methodsFor: 'accessing' stamp: 'len 2/13/2003 22:47'!
rootNode
	"return the GraphNode for the root.
	pre: roots size = 1"
	^self nodeFor: roots anyOne! !

!RootedGraph methodsFor: 'adding' stamp: ''!
buildFromRoots: rootCollection
	"Build the graph with given roots.
	The nodes should all be ImplicitGraphNodes, or have outgoing edges attached."
	self roots: rootCollection.
	self markDo: [ :junk ]! !

!RootedGraph methodsFor: 'adding' stamp: ''!
addRoot: root
	"Add root as a root of the graph."
	roots add: root.
	^nodes add: (self newNodeOn: root)! !

!RootedGraph methodsFor: 'initialization' stamp: ''!
initialize
	super initialize.
	roots := Set new! !

!RootedGraph methodsFor: 'enumerating' stamp: 'len 2/14/2003 20:06'!
walkPre: preBlock post: postBlock
	"Recursively walk the tree(s).  Apply preBlock to each node, then walk the subtree below node, then apply postBlock to the node."
	self rootNodes do: [ :root | root walkPre: preBlock post: postBlock]! !

!RootedGraph class methodsFor: 'examples' stamp: 'len 2/14/2003 23:43'!
exampleMediumTree2
	^self unordered roots: (Set with: Collection) children: [ :class | class subclasses]! !

!RootedGraph methodsFor: 'graph operations' stamp: ''!
reduceSlow
	"Remove all redundant edges to form the transitive reduction.

	Based on Algorithm 4 in 'An Algorithm for Finding a Minimum Equivalent Graph of a Digraph', by D. M. Moyles and G. L. Thompson, JACM 16(3), July 1969, pp455-60 (with obvious simplifications for acyclic graphs).
	
	This version sometimes builds huge sets!!"
	
	| seqSet |
	seqSet := Set accumulate: [ :incBlock |
				self rootNodes do: [ :root | incBlock value: (Array with: root)]].

	[seqSet isEmpty] whileFalse:
		[| newSeqSet |
"Transcript show: seqSet size printString ; cr."
		newSeqSet := Set new.
		seqSet do: [ :seq || tail |
			tail := seq last.
			tail neighborsDo: [ :neighbor |
				seq do: [ :node |
					(node ~= tail and: [node hasEdgeTo: neighbor])
						ifTrue: [
"Transcript show: node printString, ' -> ', neighbor printString ; cr."
								self removeEdgeFrom: node to: neighbor]].
				newSeqSet add: (seq copyWith: neighbor)]].
		seqSet := newSeqSet]! !

!RootedGraph class methodsFor: 'examples' stamp: 'len 2/12/2003 22:42'!
exampleSmallTree
	"RootedGraph exampleSmallTree"
	^self unordered roots: (Set with: Number) children: [ :class | class subclasses]! !

!RootedGraph methodsFor: 'adding' stamp: 'len 2/18/2003 21:38'!
roots: rootNodes childrenGenerator: aBlock label: labelBlock
	"Build the graph with given rootNodes.
	Evaluate aBlock for each node to obtain an iterator over a Collection of its children.
	Evaluate labelBlock (if not nil) for each node and child to obtain a label for the edge."
	| todo |
	self roots: rootNodes.
	todo := rootNodes asSet copy.
	[todo isEmpty] whileFalse:
		[| node |
		node := todo anyOne.
		(aBlock value: node) value:
			[ :child |
				(self includes: child) ifFalse: [todo add: child].
				labelBlock isNil
					ifTrue: [self addEdgeFrom: node to: child]
					ifFalse: [self addEdgeFrom: node to: child label: (labelBlock value: node value: child)]].
			todo remove: node].
	^self! !

!RootedGraph methodsFor: 'testing' stamp: 'len 2/15/2003 23:06'!
isCyclic
	"Is the graph cyclic?"
	| frontier remaining |
	frontier := self rootNodes.
	"Build a Bag of non-root nodes, each node once in the Bag for every predecessor."
	remaining := Bag new.
	nodes do: [ :node | remaining addAll: node neighbors].
	[| next |
	next := frontier anyIfNone: [^remaining isEmpty not].
	frontier remove: next.
	next neighborsDo:
		[ :neighbor |
			(remaining removeOne: neighbor) = 0
				ifTrue: [frontier add: neighbor]]
	] repeat! !

!RootedGraph methodsFor: 'accessing' stamp: ''!
rootNodes
	"Return the GraphNodes for the roots."
	^roots collect: [ :root | self nodeFor: root]! !

!RootedGraph methodsFor: 'adding' stamp: ''!
roots: rootNodes children: aBlock label: labelBlock
	"Build the graph with given rootNodes.
	Evaluate aBlock for each node to obtain a Collection of its children."
	^self
		roots: rootNodes 
		childrenGenerator: [ :node | [ :iterationBlock | (aBlock value: node) do: iterationBlock]]
		label: labelBlock! !

!RootedGraph methodsFor: 'enumerating' stamp: 'len 2/18/2003 21:37'!
markDo: aBlock
	"Visit each node in the graph once, applying aBlock.
	A node is only visited after at least one of its predecessors, but not necessarily after all the predecessors."
	| todo visited |
	todo := self rootNodes asSet.
	visited := Set new.
	[todo isEmpty] whileFalse:
		[| node |
		node := todo anyOne.
		visited add: node.
		aBlock value: node.
		node neighborsDo:
			[ :child |
				(visited includes: child)
					ifFalse: [todo add: child]].
		todo remove: node]! !

!RootedGraph class methodsFor: 'examples' stamp: 'len 2/12/2003 23:00'!
exampleSmallLabelledTree2
	"RootedGraph exampleSmallLabelledTree2"
	^self unorderedLabelled
		roots: (Set with: Magnitude)
		children: [ :class | class subclasses]
		label: [ :src :trg |
				trg isVariable
					ifTrue: [trg isBits ifTrue: [#variableByte] ifFalse: [#variable]]
					ifFalse: [#normal]]! !

!RootedGraph class methodsFor: 'examples' stamp: ''!
exampleImplicitSmallTree
	"RootedGraph exampleImplicitSmallTree"
	^(self implicitCollection: [ :class | class subclasses]) buildFromRoots: (Set with: Number)! !

!RootedGraph methodsFor: 'adding' stamp: ''!
roots: rootNodes childrenGenerator: aBlock
	"Build the graph with given rootNodes.
	Evaluate aBlock for each node to obtain an iterator over a Collection of its children."
	^self roots: rootNodes childrenGenerator: aBlock label: nil! !

!Graph methodsFor: 'removing' stamp: ''!
removeEdgeFrom: src to: dest
	"Remove the given edge.
	pre: the edge must be in the graph."
	^self removeEdgeFrom: src to: dest ifAbsent: [self notFoundError]! !

!Graph methodsFor: 'removing' stamp: ''!
removeEdge: edge 
	"Remove the given edge.  
	pre: the edge must be in the graph."
	self
		removeEdge: edge
		ifAbsent: [self notFoundError].
	self triggerEvent: #removeEdge: with: edge.
	type = #(#undirected )
		ifTrue: [| reversedEdge | 
			reversedEdge _ edge value -> edge key.
			self removeEdge: reversedEdge ifAbsent: [self notFoundError].
			self triggerEvent: #removeEdge: with: reversedEdge]! !

!Graph class methodsFor: 'examples' stamp: ''!
exampleGraph2
	"Graph exampleGraph2."
	| d |
	d _ self unordered.
d addEdge: #a->#b.
	^ d! !

!Graph methodsFor: 'initialization' stamp: ''!
binary
	type := #(#binary).
	nodeCreator := InitializationBlocks at: #binary! !

!Graph methodsFor: 'enumerating' stamp: ''!
do: aBlock
	self nodesDo: aBlock! !

!Graph class methodsFor: 'examples' stamp: 'Samir Saidani 6/19/2003 20:24'!
exampleUndirectedGraph
	"Graph exampleUndirectedGraph."
	| d |
	d _ self undirected.
	d addEdge: #a -> #b.
	^ d! !

!Graph methodsFor: 'initialization' stamp: ''!
unorderedLabelled
	type := #(#unordered #labelled).
	nodeCreator := InitializationBlocks at: #unorderedLabelled! !

!Graph methodsFor: 'graph operations' stamp: 'len 2/13/2003 16:56'!
distanceFrom: source to: target
	^ (self shortestPathFrom: source to: target) size! !

!Graph methodsFor: 'accessing' stamp: 'Samir Saidani 5/14/2003 16:40'!
nodes
^nodes! !

!Graph class methodsFor: 'examples' stamp: 'Samir Saidani 9/17/2003 18:49'!
exampleGraph3
	"Graph exampleGraph."
	| d |
	d _ self undirected.
	d addEdge: #r1 -> #n1.
	d addEdge: #r1 -> #n2.
	d addEdge: #r2 -> #n2.
	d addEdge: #n1 -> #n3.
	d addEdge: #n2 -> #n3.
	d addEdge: #n3 -> #r1.
	^ d! !

!Graph class methodsFor: 'instance creation' stamp: ''!
unorderedLabelled
	"Create a new labelled, unordered graph."
	^super new initialize unorderedLabelled! !

!Graph methodsFor: 'graph operations' stamp: 'len 2/13/2003 16:47'!
transposed
	| answer |
	answer _ self copyEmpty.
	self edgesDo: [:each | answer addEdge: each value -> each key].
	^ answer! !

!Graph methodsFor: 'private' stamp: ''!
removeEdgeFromNode: srcNode toNode: destNode ifAbsent: exceptionBlock
	"Remove the given edge, evaluate exceptionBlock if it doesn't exist.
	pre: both the source and target of the edge must be in the graph."
	(srcNode hasEdgeTo: destNode)
		ifTrue: [srcNode removeNeighbor: destNode.
				^self]
		ifFalse: [^exceptionBlock value]! !

!Graph methodsFor: 'copying' stamp: ''!
copyNodes: nodeBlock
	"Make a copy with the same number of nodes and edges.
	Each node is self is mapped though nodeBlock"
	^self copyNodes: nodeBlock labels: nil! !

!Graph methodsFor: 'initialization' stamp: ''!
implicitCollection: collectionBlock
	type := #(#implicitCollection).
	nodeCreator := (InitializationBlocks at: #implicitCollection) value: self value: collectionBlock! !

!Graph methodsFor: 'testing' stamp: ''!
includes: anObject 
	"Answer whether anObject is one of the receiver's elements."

	^nodes includes: anObject! !

!Graph methodsFor: 'initialization' stamp: ''!
initialize
	nodes := KeyedSet new! !

!Graph class methodsFor: 'instance creation' stamp: ''!
ordered
	"Create a new ordered graph."
	^ super new initialize ordered! !

!Graph methodsFor: 'testing' stamp: ''!
isEmpty
	"Answer whether the receiver contains any elements."

	^nodes isEmpty! !

!Graph methodsFor: 'adding' stamp: ''!
addEdgeFrom: source to: target 
	| src trgt |
	src _ self addNode: source.
	trgt _ self addNode: target.
	src addNeighbor: trgt! !

!Graph methodsFor: 'graph operations' stamp: 'len 2/15/2003 18:51'!
shortestPathFrom: source to: target
	^ Dijkstra new graph: self source: source; shortestPathTo: target! !

!Graph methodsFor: 'removing' stamp: ''!
remove: aGraphNode ifAbsent: exceptionBlock 
	"Remove the given node, evaluate exceptionBlock if it's not part of the graph.
	pre: the node shouldn't be the target of an edge."
	^nodes remove: aGraphNode ifAbsent: exceptionBlock! !

!Graph methodsFor: 'adding' stamp: ''!
addEdge: edge label: label
	"Add an edge (represented by an Association), with given label.
	pre: (self nodeFor: edge key) isLabelled"

	self addEdgeFrom: edge key to: edge value label: label.
	^edge! !

!Graph methodsFor: 'removing' stamp: ''!
removeEdgeFrom: src to: dest ifAbsent: exceptionBlock
	"Remove the given edge.
	pre: both the source and target of the edge must be in the graph."
	^self removeEdgeFromNode: (self nodeFor: src) toNode: (self nodeFor: dest) ifAbsent: exceptionBlock! !

!Graph methodsFor: 'initialization' stamp: ''!
implicitIteratorBlock: iteratorBlock
	type := #(#implicitIteratorBlock).
	nodeCreator := (InitializationBlocks at: #implicitIteratorBlock) value: self value: iteratorBlock! !

!Graph methodsFor: 'adding' stamp: ''!
add: aGraphNode 
	^nodes add: aGraphNode! !

!Graph methodsFor: 'accessing' stamp: ''!
size
	"The order of the graph."
	^nodes size! !

!Graph class methodsFor: 'examples' stamp: ''!
exampleGraph
	"Graph exampleGraph."
	| d |
	d _ self ordered.
	d addEdge: #r1 -> #n1.
	d addEdge: #r1 -> #n2.
	d addEdge: #r2 -> #n2.
	d addEdge: #n1 -> #n3.
	d addEdge: #n2 -> #n3.
	d addEdge: #n3 -> #r1.
	^ d! !

!Graph methodsFor: 'private' stamp: ''!
nodeFor: object	
	^nodes at: object! !

!Graph methodsFor: 'private' stamp: ''!
isLabelled
	"Is the node creator for labelled nodes?"
	^type includes: #labelled! !

!Graph class methodsFor: 'instance creation' stamp: ''!
binaryLabelled
	"Create an labelled, ordered graph with arity of 2 (ie each vertex has exactly two connecting edges)."
	^super new initialize binaryLabelled! !

!Graph methodsFor: 'graph operations' stamp: 'len 2/14/2003 22:56'!
componentsDo: aBlock
	"Enumerate the Strongly Connected Components of the receiver."
	| component remaining node |
	remaining _ self asSet.
	[remaining isEmpty]
		whileFalse: [node _ remaining anyOne.
					component _ self copyEmpty.
					node markDo: [:each |
						component add: each.
						remaining remove: each ifAbsent: []].
					aBlock value: component]! !

!Graph methodsFor: 'private' stamp: ''!
nodeCreator: n
	nodeCreator := n! !

!Graph class methodsFor: 'instance creation' stamp: ''!
unordered
	"Create a new unordered graph."
	^super new initialize unordered! !

!Graph class methodsFor: 'instance creation' stamp: ''!
binary
	"Create an ordered graph with arity of 2 (ie each vertex has exactly two connecting edges)."
	^super new initialize binary! !

!Graph methodsFor: 'private' stamp: ''!
newNodeOn: object
	^nodeCreator value: object! !

!Graph class methodsFor: 'examples' stamp: ''!
exampleImplicitGraph2
	"Graph exampleImplicitGraph2"
	| tree |
	tree := self implicitIteratorBlock: [ :class | [ :aBlock | class subclasses do: aBlock]].
	Number withAllSubclasses do: [ :class | tree addNode: class].
	^tree! !

!Graph methodsFor: 'initialization' stamp: ''!
binaryLabelled
	type := #(#binary #labelled).
	nodeCreator := InitializationBlocks at: #binaryLabelled! !

!Graph methodsFor: 'copying' stamp: ''!
copyNodes: nodeBlock  labels: labelBlock
	"Make a copy with the same number of nodes and edges.
	Each node is self is mapped though nodeBlock, labels (if used) are
	mapped through labelBlock (if non-nil)."
	| newGraph map |
	newGraph := self copyEmpty.
	map := DictionaryWithDefault newWithDefaultValueBlock: nodeBlock.
	self do: [ :node |
		node neighborsAndLabelsDo: [ :n :label |
			(labelBlock notNil and: [label notNil])
				ifTrue: [newGraph 
							addEdgeFrom: (map at: node value)
							to: (map at: n value)
							label: (labelBlock value: label)]
				ifFalse: [newGraph
							addEdgeFrom: (map at: node value)
							to: (map at: n value)]]].
	^newGraph! !

!Graph methodsFor: 'accessing' stamp: ''!
order
	^nodes size! !
Graph initialize!
